# -*- coding: utf-8 -*-

from django.conf import settings
from django.forms import TextInput
from django.template.loader import render_to_string


class ColorWidget(TextInput):

    template_name = "colorfield/color.html"

    class Media:
        if settings.DEBUG:
            js = [
                "colorfield/jscolor/jscolor.js",
                "colorfield/colorfield.js",
            ]
        else:
            js = [
                "colorfield/jscolor/jscolor.min.js",
                "colorfield/colorfield.js",
            ]

    def get_context(self, name, value, attrs=None):
        context = {}
        context.update(self.attrs.copy() or {})
        context.update(attrs or {})
        context.update(
            {
                "widget": self,
                "name": name,
                "value": value,
            }
        )
        return context

    def render(self, name, value, attrs=None, renderer=None):
        return render_to_string(
            self.template_name, self.get_context(name, value, attrs)
        )



# -*- coding: utf-8 -*-

from django.core.exceptions import ValidationError as DjangoValidationError

try:
    from rest_framework.serializers import (
        CharField,
        ValidationError as DRFValidationError,
    )
except ImportError:
    ModuleNotFoundError("Django REST Framework is not installed.")

from colorfield.validators import color_hex_validator, color_hexa_validator


class ColorField(CharField):

    default_error_messages = {
        "invalid": [
            color_hex_validator.message,
            color_hexa_validator.message,
        ]
    }

    def to_internal_value(self, data):
        has_hex_error = False
        has_hexa_error = False
        try:
            color_hex_validator(data)
        except DjangoValidationError:
            has_hex_error = True

        try:
            color_hexa_validator(data)
        except DjangoValidationError:
            has_hexa_error = True

        if has_hex_error and has_hexa_error:
            raise DRFValidationError(self.default_error_messages.get("invalid"))

        return data



# -*- coding: utf-8 -*-

from PIL import Image


def get_image_background_color(img, alpha=False):
    img = img.convert("RGBA" if alpha else "RGB")
    pixel_color = img.getpixel((1, 1))
    color_format = "#" + "%02x" * len(pixel_color)
    color = color_format % pixel_color
    color = color.upper()
    return color


def get_image_file_background_color(img_file, alpha=False):
    color = ""
    with Image.open(img_file) as image:
        color = get_image_background_color(image, alpha)
    return color



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

import os, sys

exec(open("colorfield/version.py").read())

github_url = "https://github.com/fabiocaccamo"
sponsor_url = "https://github.com/sponsors/fabiocaccamo/"
twitter_url = "https://twitter.com/fabiocaccamo"
package_name = "django-colorfield"
package_url = "{}/{}".format(github_url, package_name)
package_path = os.path.abspath(os.path.dirname(__file__))
long_description_file_path = os.path.join(package_path, "README.md")
long_description_content_type = "text/markdown"
long_description = ""
try:
    long_description_file_options = (
        {} if sys.version_info[0] < 3 else {"encoding": "utf-8"}
    )
    with open(long_description_file_path, "r", **long_description_file_options) as f:
        long_description = f.read()
except IOError:
    pass

setup(
    name=package_name,
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    include_package_data=True,
    version=__version__,
    description="simple color field for your models with a nice color-picker in the admin-interface.",
    long_description=long_description,
    long_description_content_type=long_description_content_type,
    author="Jared Forsyth, Fabio Caccamo",
    author_email="jared@jaredforsyth.com, fabio.caccamo@gmail.com",
    url=package_url,
    download_url="{}/archive/{}.tar.gz".format(package_url, __version__),
    project_urls={
        "Documentation": "{}#readme".format(package_url),
        "Issues": "{}/issues".format(package_url),
        "Funding": sponsor_url,
        "Twitter": twitter_url,
    },
    keywords=[
        "django",
        "colorfield",
        "colorpicker",
        "color",
        "field",
        "picker",
        "chooser",
        "admin",
        "python",
    ],
    requires=["django (>=1.7)"],
    install_requires=["Pillow (>=5.0)"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.7",
        "Framework :: Django :: 1.8",
        "Framework :: Django :: 1.9",
        "Framework :: Django :: 1.10",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Build Tools",
    ],
    license="MIT",
    test_suite="runtests.runtests",
)



from re import compile as re_compile

from django import VERSION as DJANGO_VERSION
from django.core.validators import RegexValidator

if DJANGO_VERSION >= (2, 0):
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _


COLOR_HEX_RE = re_compile("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$")
color_hex_validator = RegexValidator(
    COLOR_HEX_RE, _("Enter a valid hex color, eg. #000000"), "invalid"
)


COLOR_HEXA_RE = re_compile("#([A-Fa-f0-9]{8}|[A-Fa-f0-9]{4})$")
color_hexa_validator = RegexValidator(
    COLOR_HEXA_RE, _("Enter a valid hexa color, eg. #00000000"), "invalid"
)



# -*- coding: utf-8 -*-

from django import VERSION as DJANGO_VERSION

from colorfield.utils import get_image_file_background_color
from colorfield.validators import color_hex_validator, color_hexa_validator
from colorfield.widgets import ColorWidget

if DJANGO_VERSION >= (1, 8):
    from django.core.exceptions import FieldDoesNotExist
else:
    FieldDoesNotExist = Exception
from django.core.exceptions import ImproperlyConfigured

from django.db.models import CharField, signals
from django.db.models.fields.files import ImageField


VALIDATORS_PER_FORMAT = {"hex": color_hex_validator, "hexa": color_hexa_validator}

DEFAULT_PER_FORMAT = {"hex": "#FFFFFF", "hexa": "#FFFFFFFF"}


class ColorField(CharField):

    default_validators = []

    def __init__(self, *args, **kwargs):
        # works like Django choices, but does not restrict input to the given choices
        self.samples = kwargs.pop("samples", None)
        self.format = kwargs.pop("format", "hex").lower()
        if self.format not in ["hex", "hexa"]:
            raise ValueError("Unsupported color format: {}".format(self.format))
        self.default_validators = [VALIDATORS_PER_FORMAT[self.format]]

        self.image_field = kwargs.pop("image_field", None)
        if self.image_field:
            kwargs.setdefault("blank", True)

        kwargs.setdefault("max_length", 18)
        if kwargs.get("null"):
            kwargs.setdefault("blank", True)
            kwargs.setdefault("default", None)
        elif kwargs.get("blank"):
            kwargs.setdefault("default", "")
        else:
            kwargs.setdefault("default", DEFAULT_PER_FORMAT[self.format])
        super(ColorField, self).__init__(*args, **kwargs)

        if self.choices and self.samples:
            raise ImproperlyConfigured(
                "Invalid options: 'choices' and 'samples' are mutually exclusive, "
                "you can set only one of the two for a ColorField instance."
            )

    def formfield(self, **kwargs):
        palette = []
        if self.choices:
            choices = self.get_choices(include_blank=False)
            palette = [choice[0] for choice in choices]
        elif self.samples:
            palette = [choice[0] for choice in self.samples]
        kwargs["widget"] = ColorWidget(
            attrs={
                "default": self.get_default(),
                "format": self.format,
                "palette": palette,
                # # this will be used to hide the widget color spectrum if choices is defined:
                # 'palette_choices_only': bool(self.choices),
            }
        )
        return super(ColorField, self).formfield(**kwargs)

    def contribute_to_class(self, cls, name, **kwargs):
        super(ColorField, self).contribute_to_class(cls, name, **kwargs)
        if cls._meta.abstract:
            return
        if self.image_field:
            signals.post_save.connect(self._update_from_image_field, sender=cls)

    def deconstruct(self):
        name, path, args, kwargs = super(ColorField, self).deconstruct()
        kwargs["samples"] = self.samples
        kwargs["image_field"] = self.image_field
        return name, path, args, kwargs

    def _get_image_field_color(self, instance):
        color = ""
        image_file = getattr(instance, self.image_field)
        if image_file:
            alpha = self.format == "hexa"
            if DJANGO_VERSION >= (2, 0):
                # https://stackoverflow.com/a/3033986/2096218
                with image_file.open() as _:
                    color = get_image_file_background_color(image_file, alpha)
            else:
                # https://stackoverflow.com/a/3033986/2096218
                image_file.open()
                color = get_image_file_background_color(image_file, alpha)
                image_file.close()
        return color

    def _update_from_image_field(self, instance, created, *args, **kwargs):
        if not instance or not instance.pk or not self.image_field:
            return
        # check if the field is a valid ImageField
        try:
            field_cls = instance._meta.get_field(self.image_field)
            if not isinstance(field_cls, ImageField):
                raise ImproperlyConfigured(
                    'Invalid "image_field" field type, '
                    'expected an instance of "models.ImageField".'
                )
        except FieldDoesNotExist as _:
            raise ImproperlyConfigured(
                'Invalid "image_field" field name, '
                '"{}" field not found.'.format(self.image_field)
            )
        # update value from picking color from image field
        color = self._get_image_field_color(instance)
        color_field_name = self.attname
        color_field_value = getattr(instance, color_field_name, None)
        if color_field_value != color:
            color_field_value = color or self.default
            # update in-memory value
            setattr(instance, color_field_name, color_field_value)
            # update stored value
            manager = instance.__class__.objects
            manager.filter(pk=instance.pk).update(
                **{color_field_name: color_field_value}
            )
